SystemOrganization addCategory: #BookStore!

Exception subclass: #CanNotAddANegativeAmountOfBooksToCart
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BookStore'!

Exception subclass: #CartIsNotValid
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BookStore'!

Object subclass: #Cart
	instanceVariableNames: 'content expirationDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BookStore'!

!Cart methodsFor: 'adding' stamp: 'GuillermoAres 6/12/2012 22:46'!
add: anISBN 
	
	self checkIfExpired.
	content add: anISBN.! !

!Cart methodsFor: 'adding' stamp: 'GuillermoAres 6/12/2012 22:46'!
add: anAmount booksWithISBN: anISBN
	
	self checkIfExpired.
	(anAmount <= 0) ifTrue: [^CanNotAddANegativeAmountOfBooksToCart signal ].
	content add: anISBN withOccurrences: anAmount.! !

!Cart methodsFor: 'private' stamp: 'GuillermoAres 6/12/2012 22:48'!
checkIfExpired
	
	(self isValid) ifFalse: [CartIsNotValid signal].! !

!Cart methodsFor: 'accessing' stamp: 'GuillermoAres 6/12/2012 20:26'!
content

	^ content! !

!Cart methodsFor: 'private' stamp: 'GuillermoAres 6/12/2012 22:45'!
expirationDate: aDateAndTime 

	expirationDate := aDateAndTime! !

!Cart methodsFor: 'initialization' stamp: 'GuillermoAres 6/12/2012 22:36'!
initialize

	content := Bag new.
	expirationDate := DateAndTime now + (Duration minutes:30).! !

!Cart methodsFor: 'testing' stamp: 'GuillermoAres 6/12/2012 22:47'!
isEmpty
	
	self checkIfExpired.
	^self content size = 0.! !

!Cart methodsFor: 'testing' stamp: 'GuillermoAres 6/12/2012 22:37'!
isValid
	
	^DateAndTime now < expirationDate.! !

TestCase subclass: #TestsCart
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BookStore'!

!TestsCart methodsFor: 'exceptions' stamp: 'GuillermoAres 6/12/2012 22:04'!
canNotAddANegativeAmountOfBooksToCart

	^CanNotAddANegativeAmountOfBooksToCart! !

!TestsCart methodsFor: 'domain entities' stamp: 'GuillermoAres 6/12/2012 20:24'!
cart
	
	^Cart new.! !

!TestsCart methodsFor: 'exceptions' stamp: 'GuillermoAres 6/12/2012 23:02'!
cartIsNotValid
	
	^CartIsNotValid.! !

!TestsCart methodsFor: 'domain entities' stamp: 'GuillermoAres 6/12/2012 22:45'!
invalidCart
	
	^Cart new expirationDate: DateAndTime now.! !

!TestsCart methodsFor: 'domain entities' stamp: 'GuillermoAres 6/12/2012 22:05'!
isbn

	^1234.! !

!TestsCart methodsFor: 'tests' stamp: 'GuillermoAres 6/12/2012 20:28'!
test01WhenACartIsCreatedThenItIsEmpty

	self assert: self cart isEmpty = true.! !

!TestsCart methodsFor: 'tests' stamp: 'GuillermoAres 6/12/2012 22:03'!
test02WhenANegativeNumberOfISBNsAreAddedToACartThenAnExceptionIsRaised

	self should: [ self cart add: -5 booksWithISBN: self isbn] raise: self canNotAddANegativeAmountOfBooksToCart.! !

!TestsCart methodsFor: 'tests' stamp: 'GuillermoAres 6/12/2012 22:23'!
test03IfABookIsAddedToTheCartThenItIsNotEmpty

	self deny: (self cart add: self isbn; isEmpty).! !

!TestsCart methodsFor: 'tests' stamp: 'GuillermoAres 6/12/2012 23:03'!
test04WhenACartIsCreatedThenItIsValid

	self assert: self cart isValid.! !

!TestsCart methodsFor: 'tests' stamp: 'GuillermoAres 6/12/2012 22:59'!
test05AnyOperationWithAnInvalidCartRaisesACartIsNotValidException

	self should: [ self invalidCart add: self isbn] raise: self cartIsNotValid.
	self should: [ self invalidCart add: 3 booksWithISBN: self isbn] raise: self cartIsNotValid.
	self should: [ self invalidCart add: -3 booksWithISBN: self isbn] raise: self cartIsNotValid.
	self should: [ self invalidCart isEmpty] raise: self cartIsNotValid.! !
